
/*
 *    ChunkSync - create space-efficient incremental (remote) backups of large files
 *    Copyright (C) 2009  Florian Zumbiehl <florz@florz.de>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 *    In addition, as a special exception, the author of this
 *    program gives permission to link the code of its 
 *    release with the OpenSSL project's "OpenSSL" library (or
 *    with modified versions of it that use the same license as
 *    the "OpenSSL" library), and distribute the linked
 *    executables. You must obey the GNU General Public 
 *    License in all respects for all of the code used other 
 *    than "OpenSSL".  If you modify this file, you may extend
 *    this exception to your version of the file, but you are
 *    not obligated to do so.  If you do not wish to do so,
 *    delete this exception statement from your version.
 */

#include <stdbool.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "defs.h"
#include "io.h"
#include "err.h"
#include "sender.h"
#include "receiver.h"
#include "backend.h"

struct xfh stdio_fh;

void backend_main(bool sender_mode,int argc){
	char buf[strlen(PROTOCOL_MAGIC)],*path;

	if(argc)usage(true);
	xfh_init(&stdio_fh,STDIN_FILENO,STDOUT_FILENO,NULL,"frontend");
	xwrite(&stdio_fh,PROTOCOL_MAGIC,strlen(PROTOCOL_MAGIC));
	xread(&stdio_fh,buf,strlen(PROTOCOL_MAGIC));
	if(memcmp(buf,PROTOCOL_MAGIC,strlen(PROTOCOL_MAGIC)))die(false,"incorrect protocol magic from frontend");
	xwrite_u8(&stdio_fh,0);
	xwrite_u8(&stdio_fh,0);
	if(xread_u8(&stdio_fh))die(false,"incompatible protocol version from frontend");
	xread_u8(&stdio_fh);
	if(xread_u8(&stdio_fh))proterr();
	path=xread_string(&stdio_fh);
	if(sender_mode)sender_main(path);
		else receiver_main(path);
	free(path);
}

