
/*
 *    ChunkSync - create space-efficient incremental (remote) backups of large files
 *    Copyright (C) 2009  Florian Zumbiehl <florz@florz.de>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 *    In addition, as a special exception, the author of this
 *    program gives permission to link the code of its 
 *    release with the OpenSSL project's "OpenSSL" library (or
 *    with modified versions of it that use the same license as
 *    the "OpenSSL" library), and distribute the linked
 *    executables. You must obey the GNU General Public 
 *    License in all respects for all of the code used other 
 *    than "OpenSSL".  If you modify this file, you may extend
 *    this exception to your version of the file, but you are
 *    not obligated to do so.  If you do not wish to do so,
 *    delete this exception statement from your version.
 */

#ifndef IO_H
#define IO_H

#include <stdbool.h>
#include <stdint.h>
#include <sys/types.h>

struct xfh {
	int r,w;
	void (*graceful)(bool);
	const char *name;
};

void xfh_init(struct xfh *,int,int,void (*)(bool),const char *);
void xfh_close(const struct xfh *);

void xwrite(const struct xfh *,const void *,size_t);
void xwrite_u8(const struct xfh *,uint8_t);
void xwrite_u32(const struct xfh *,uint32_t);
void xwrite_u64(const struct xfh *,uint64_t);
void xwrite_string(const struct xfh *,const char *);

void xread(const struct xfh *,void *,size_t);
uint8_t xread_u8(const struct xfh *);
uint32_t xread_u32(const struct xfh *);
uint64_t xread_u64(const struct xfh *);
char *xread_string(const struct xfh *);

void xcopy(const struct xfh *,const struct xfh *,off_t);

#endif

