
/*
 *    ChunkSync - create space-efficient incremental (remote) backups of large files
 *    Copyright (C) 2009  Florian Zumbiehl <florz@florz.de>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 *    In addition, as a special exception, the author of this
 *    program gives permission to link the code of its 
 *    release with the OpenSSL project's "OpenSSL" library (or
 *    with modified versions of it that use the same license as
 *    the "OpenSSL" library), and distribute the linked
 *    executables. You must obey the GNU General Public 
 *    License in all respects for all of the code used other 
 *    than "OpenSSL".  If you modify this file, you may extend
 *    this exception to your version of the file, but you are
 *    not obligated to do so.  If you do not wish to do so,
 *    delete this exception statement from your version.
 */

#include <stdbool.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>

#include "err.h"
#include "defs.h"
#include "frontend.h"
#include "backend.h"

int main(int argc,char **argv){
	bool validation_error=false,show_version=false,show_help=false,backend_mode=false,backend_sender=false;
	int opt;
	size_t linkdirs_cnt=0;
	char *linkdirs_names[LINKDIRS_MAX];

	err_set_progname(*argv);
	while((opt=getopt(argc,argv,":hVb:l:"))!=-1)
		switch(opt){
			case 'h':
				show_help=true;
				break;
			case 'V':
				show_version=true;
				break;
			case 'b':
				if((backend_sender=strcmp(optarg,"receiver"))&&strcmp(optarg,"sender")){
					warn(false,"invalid argument to -b");
					validation_error=true;
				}else backend_mode=true;
				break;
			case 'l':
				if(linkdirs_cnt==LINKDIRS_MAX){
					warn(false,"more than %i link directories specified",LINKDIRS_MAX);
					validation_error=true;
				}else linkdirs_names[linkdirs_cnt++]=optarg;
				break;
			case '?':
				warn(false,"invalid option: -%c",optopt);
				validation_error=true;
				break;
			case ':':
				warn(false,"option requires an argument: -%c",optopt);
				validation_error=true;
				break;
		}
	argc-=optind;
	if(show_help+show_version+backend_mode+!!linkdirs_cnt>1||((show_help||show_version)&&argc))validation_error=true;
	if(validation_error||show_help){
		usage(validation_error);
	}else if(show_version){
		warn(false,"ChunkSync v" VERSION);
	}else{
		err_set_backend_mode(backend_mode);
		if(signal(SIGPIPE,SIG_IGN)==SIG_ERR)die(true,"signal()");
		if(backend_mode)backend_main(backend_sender,argc);
			else frontend_main(linkdirs_cnt,linkdirs_names,argc,argv+optind);
	}
	exit(0);
}

